@@ -31,14 +31,17 @@
 
 #include <string.h>
 #include <glib/gi18n-lib.h>
+#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtkalignment.h> 
 #include <gtk/gtkbox.h> 
 #include <gtk/gtkbutton.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkcellrenderertoggle.h>
+#include <gtk/gtkcellrendererpixbuf.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkhbbox.h>
 #include <gtk/gtkimage.h>
+#include <gtk/gtkicontheme.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkliststore.h>
 #include <gtk/gtkscrolledwindow.h>
@@ -68,6 +71,7 @@ struct _EelMimeApplicationChooserDetails {
 
 enum {
 	COLUMN_DEFAULT,
+	COLUMN_ICON,
 	COLUMN_NAME,
 	COLUMN_ID,
 	NUM_COLUMNS
@@ -86,6 +90,7 @@ eel_mime_application_chooser_finalize (GObject *object)
 
 	g_free (chooser->details->uri);
 	g_free (chooser->details->mime_type);
+	g_free (chooser->details->real_mime_type);
 	g_free (chooser->details->mime_description);
 	
 	g_free (chooser->details);
@@ -141,7 +146,7 @@ default_toggled_cb (GtkCellRendererToggle *renderer,
 				    COLUMN_ID, &id,
 				    -1);
 		
-		if (!is_default) {
+		if (!is_default && id != NULL) {
 			eel_mime_set_default_application (chooser->details->mime_type,
 							  id);
 			refresh_model (chooser);
@@ -208,6 +213,7 @@ create_tree_view (EelMimeApplicationChooser *chooser)
 	
 	store = gtk_list_store_new (NUM_COLUMNS,
 				    G_TYPE_BOOLEAN,
+				    GDK_TYPE_PIXBUF,
 				    G_TYPE_STRING,
 				    G_TYPE_STRING);
 	gtk_tree_view_set_model (GTK_TREE_VIEW (treeview),
@@ -228,6 +234,14 @@ create_tree_view (EelMimeApplicationChooser *chooser)
 							   NULL);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
 
+	renderer = gtk_cell_renderer_pixbuf_new ();
+	column = gtk_tree_view_column_new_with_attributes (_("Icon"),
+							   renderer,
+							   "pixbuf",
+							   COLUMN_ICON,
+							   NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
+
 	chooser->details->toggle_renderer = renderer;
 	renderer = gtk_cell_renderer_text_new ();
 	column = gtk_tree_view_column_new_with_attributes (_("Name"),
@@ -265,7 +279,7 @@ static void
 remove_clicked_cb (GtkButton *button, 
 		   gpointer user_data)
 {
-	const char *id;
+	char *id;
 	EelMimeApplicationChooser *chooser;
 	
 	chooser = EEL_MIME_APPLICATION_CHOOSER (user_data);
@@ -275,6 +289,7 @@ remove_clicked_cb (GtkButton *button,
 	if (id) {
 		eel_mime_application_remove (id);
 		refresh_model (chooser);
+		g_free (id);
 	}
 }
 
@@ -379,6 +394,10 @@ refresh_model (EelMimeApplicationChooser *chooser)
 	GnomeVFSMimeApplication *default_app;
 	GList *l;
 	GtkTreeSelection *selection;
+	GtkTreeViewColumn *column;
+
+	column = gtk_tree_view_get_column (GTK_TREE_VIEW (chooser->details->treeview), 0);
+	gtk_tree_view_column_set_visible (column, TRUE);
 	
 	gtk_list_store_clear (chooser->details->model);
 
@@ -391,20 +410,56 @@ refresh_model (EelMimeApplicationChooser *chooser)
 		gboolean is_default;
 		GnomeVFSMimeApplication *application;
 		char *escaped;
+		const char *icon;
+		GdkPixbuf *pixbuf;
+
+		pixbuf = NULL;
 
 		application = l->data;
 		
-		is_default = default_app && !strcmp (default_app->id, application->id);
+		is_default = default_app &&
+			     gnome_vfs_mime_application_equal (default_app, application);
 
 		escaped = g_markup_escape_text (application->name, -1);
 
+		icon = gnome_vfs_mime_application_get_icon (application);
+
+		if (icon != NULL) {
+			if (g_path_is_absolute (icon)) {
+				pixbuf = gdk_pixbuf_new_from_file_at_size (icon, 24, 24, NULL);
+			}
+			if (pixbuf == NULL) {
+				pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+								   icon, 24, 0, NULL);
+			}
+			if (pixbuf == NULL && strrchr (icon, '.') != NULL) {
+				char *name;
+				name = g_strndup (icon, strlen (icon) - strlen (strrchr (icon, '.')));
+				pixbuf = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+								   name, 24, 0, NULL);
+				g_free (name);
+			}
+	
+			if (pixbuf == NULL) {
+				char *filename;
+				filename = g_strdup_printf (DATADIR"/pixmaps/%s", icon);
+				pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 24, 24, NULL);
+				g_free (filename);
+			}
+		}
+
 		gtk_list_store_append (chooser->details->model, &iter);
 		gtk_list_store_set (chooser->details->model, &iter,
 				    COLUMN_DEFAULT, is_default,
+				    COLUMN_ICON, pixbuf,
 				    COLUMN_NAME, escaped,
-				    COLUMN_ID, application->id, 
+				    COLUMN_ID,
+				    gnome_vfs_mime_application_get_desktop_id (application), 
 				    -1);
+
 		g_free (escaped);
+		if (pixbuf != NULL)
+			g_object_unref(pixbuf);
 	}
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (chooser->details->treeview));
@@ -417,9 +472,7 @@ refresh_model (EelMimeApplicationChooser *chooser)
 	} else {
 		GtkTreeIter iter;
 
-		g_object_set (chooser->details->toggle_renderer,
-			      "visible", FALSE,
-			      NULL);
+		gtk_tree_view_column_set_visible (column, FALSE);
 		gtk_list_store_append (chooser->details->model, &iter);
 		gtk_list_store_set (chooser->details->model, &iter,
 				    COLUMN_NAME, _("<i>No applications selected</i>"),
@@ -465,6 +518,7 @@ set_uri_and_mime_type (EelMimeApplicationChooser *chooser,
 		chooser->details->mime_type = 
 			g_strdup_printf ("application/x-extension-%s", 
 					 extension);
+		/* the %s here is a file extension */
 		chooser->details->mime_description = 
 			g_strdup_printf (_("%s document"), extension);
 		
@@ -482,6 +536,7 @@ set_uri_and_mime_type (EelMimeApplicationChooser *chooser,
 		chooser->details->mime_description = description;
 	}
 
+	/* first %s is filename, second %s is mime-type description */
 	label = g_strdup_printf (_("Select an application to open <i>%s</i> and others of type \"%s\""), name, chooser->details->mime_description);
 	
 	gtk_label_set_markup (GTK_LABEL (chooser->details->label), label);
